@@ -90,7 +90,6 @@ typedef struct {
   StunTransactionId id;
   GSource *source;
   TurnPriv *priv;
-  gint ref;
 } SendRequest;
 
 static void socket_close (NiceSocket *sock);
@@ -200,7 +199,12 @@ socket_close (NiceSocket *sock)
     SendRequest *r = i->data;
     g_source_destroy (r->source);
     g_source_unref (r->source);
+    r->source = NULL;
+
+    stun_agent_forget_transaction (&priv->agent, r->id);
+
     g_slice_free (SendRequest, r);
+
   }
   g_queue_free (priv->send_requests);
 
@@ -316,7 +320,6 @@ socket_send (NiceSocket *sock, const NiceAddress *to,
       req->source = agent_timeout_add_with_context (priv->nice, STUN_END_TIMEOUT,
           priv_forget_send_request, req);
       g_queue_push_tail (priv->send_requests, req);
-      g_atomic_int_inc (&req->ref);
     }
   }
 
@@ -340,24 +343,26 @@ priv_forget_send_request (gpointer pointer)
 {
   SendRequest *req = pointer;
 
-  g_atomic_int_inc (&req->ref);
+  agent_lock ();
 
-  g_static_rec_mutex_lock (&req->priv->nice->mutex);
+  if (g_source_is_destroyed (g_main_current_source ())) {
+    nice_debug ("Source was destroyed. "
+        "Avoided race condition in turn.c:priv_forget_send_request");
+    agent_unlock ();
+    return FALSE;
+  }
 
   stun_agent_forget_transaction (&req->priv->agent, req->id);
 
+  g_queue_remove (req->priv->send_requests, req);
+
   g_source_destroy (req->source);
   g_source_unref (req->source);
+  req->source = NULL;
 
-  if (g_queue_index (req->priv->send_requests, req) != -1) {
-    g_queue_remove (req->priv->send_requests, req);
-    (void)g_atomic_int_dec_and_test (&req->ref);
-  }
+  agent_unlock ();
 
-  g_static_rec_mutex_unlock (&req->priv->nice->mutex);
-
-  if (g_atomic_int_dec_and_test (&req->ref))
-      g_slice_free (SendRequest, req);
+  g_slice_free (SendRequest, req);
 
   return FALSE;
 }
@@ -410,11 +415,11 @@ nice_turn_socket_parse_recv (NiceSocket *sock, NiceSocket **from_sock,
           if (req) {
             g_source_destroy (req->source);
             g_source_unref (req->source);
+            req->source = NULL;
 
             g_queue_remove (priv->send_requests, req);
 
-            if (g_atomic_int_dec_and_test (&req->ref))
-              g_slice_free (SendRequest, req);
+            g_slice_free (SendRequest, req);
           }
 
           if (priv->compatibility == NICE_TURN_SOCKET_COMPATIBILITY_GOOGLE) {
@@ -641,9 +646,23 @@ priv_retransmissions_tick (gpointer pointer)
   TurnPriv *priv = pointer;
   gboolean ret;
 
-  g_static_rec_mutex_lock (&priv->nice->mutex);
+  agent_lock ();
+  if (g_source_is_destroyed (g_main_current_source ())) {
+    nice_debug ("Source was destroyed. "
+        "Avoided race condition in turn.c:priv_retransmissions_tick");
+    agent_unlock ();
+    return FALSE;
+  }
+
   ret = priv_retransmissions_tick_unlocked (priv);
-  g_static_rec_mutex_unlock (&priv->nice->mutex);
+  if (ret == FALSE) {
+    if (priv->tick_source != NULL) {
+      g_source_destroy (priv->tick_source);
+      g_source_unref (priv->tick_source);
+      priv->tick_source = NULL;
+    }
+  }
+  agent_unlock ();
 
   return ret;
 }
