@@ -59,6 +59,7 @@ void stun_agent_init (StunAgent *agent, const uint16_t *known_attributes,
   agent->known_attributes = (uint16_t *) known_attributes;
   agent->compatibility = compatibility;
   agent->usage_flags = usage_flags;
+  agent->software_attribute = NULL;
 
   for (i = 0; i < STUN_AGENT_MAX_SAVED_IDS; i++) {
     agent->sent_ids[i].valid = FALSE;
@@ -319,6 +320,21 @@ StunValidationStatus stun_agent_validate (StunAgent *agent, StunMessage *msg,
 
 }
 
+bool stun_agent_forget_transaction (StunAgent *agent, StunTransactionId id)
+{
+  int i;
+
+  for (i = 0; i < STUN_AGENT_MAX_SAVED_IDS; i++) {
+    if (agent->sent_ids[i].valid == TRUE &&
+        memcmp (id, agent->sent_ids[i].id,
+            sizeof(StunTransactionId)) == 0) {
+      agent->sent_ids[i].valid = FALSE;
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
 
 bool stun_agent_init_request (StunAgent *agent, StunMessage *msg,
     uint8_t *buffer, size_t buffer_len, StunMethod m)
@@ -343,6 +359,12 @@ bool stun_agent_init_request (StunAgent *agent, StunMessage *msg,
       uint32_t cookie = htonl (STUN_MAGIC_COOKIE);
       memcpy (msg->buffer + STUN_MESSAGE_TRANS_ID_POS, &cookie, sizeof (cookie));
     }
+    if ((agent->compatibility == STUN_COMPATIBILITY_RFC5389 ||
+            agent->compatibility == STUN_COMPATIBILITY_WLM2009) &&
+        (agent->software_attribute != NULL ||
+            agent->usage_flags & STUN_AGENT_USAGE_ADD_SOFTWARE)) {
+      stun_message_append_software (msg, agent->software_attribute);
+    }
   }
 
   return ret;
@@ -403,8 +425,9 @@ bool stun_agent_init_response (StunAgent *agent, StunMessage *msg,
 
     if ((agent->compatibility == STUN_COMPATIBILITY_RFC5389 ||
             agent->compatibility == STUN_COMPATIBILITY_WLM2009) &&
-      agent->usage_flags & STUN_AGENT_USAGE_ADD_SOFTWARE) {
-      stun_message_append_software (msg);
+        (agent->software_attribute != NULL ||
+            agent->usage_flags & STUN_AGENT_USAGE_ADD_SOFTWARE)) {
+      stun_message_append_software (msg, agent->software_attribute);
     }
     return TRUE;
   }
@@ -439,8 +462,9 @@ bool stun_agent_init_error (StunAgent *agent, StunMessage *msg,
 
     if ((agent->compatibility == STUN_COMPATIBILITY_RFC5389 ||
             agent->compatibility == STUN_COMPATIBILITY_WLM2009) &&
-      agent->usage_flags & STUN_AGENT_USAGE_ADD_SOFTWARE) {
-      stun_message_append_software (msg);
+        (agent->software_attribute != NULL ||
+            agent->usage_flags & STUN_AGENT_USAGE_ADD_SOFTWARE)) {
+      stun_message_append_software (msg, agent->software_attribute);
     }
     if (stun_message_append_error (msg, err) == STUN_MESSAGE_RETURN_SUCCESS) {
       return TRUE;
@@ -485,10 +509,21 @@ size_t stun_agent_finish_message (StunAgent *agent, StunMessage *msg,
 {
   uint8_t *ptr;
   uint32_t fpr;
-  int i;
-  StunTransactionId id;
+  int saved_id_idx = 0;
   uint8_t md5[16];
 
+  if (stun_message_get_class (msg) == STUN_REQUEST) {
+    for (saved_id_idx = 0; saved_id_idx < STUN_AGENT_MAX_SAVED_IDS; saved_id_idx++) {
+      if (agent->sent_ids[saved_id_idx].valid == FALSE) {
+        break;
+      }
+    }
+  }
+  if (saved_id_idx == STUN_AGENT_MAX_SAVED_IDS) {
+    stun_debug ("Saved ids full");
+    return 0;
+  }
+
   if (msg->key != NULL) {
     key = msg->key;
     key_len = msg->key_len;
@@ -587,20 +622,14 @@ size_t stun_agent_finish_message (StunAgent *agent, StunMessage *msg,
 
 
   if (stun_message_get_class (msg) == STUN_REQUEST) {
-    for (i = 0; i < STUN_AGENT_MAX_SAVED_IDS; i++) {
-      if (agent->sent_ids[i].valid == FALSE) {
-        stun_message_id (msg, id);
-        memcpy (agent->sent_ids[i].id, id, sizeof(StunTransactionId));
-        agent->sent_ids[i].method = stun_message_get_method (msg);
-        agent->sent_ids[i].key = (uint8_t *) key;
-        agent->sent_ids[i].key_len = key_len;
-        memcpy (agent->sent_ids[i].long_term_key, msg->long_term_key,
-            sizeof(msg->long_term_key));
-        agent->sent_ids[i].long_term_valid = msg->long_term_valid;
-        agent->sent_ids[i].valid = TRUE;
-        break;
-      }
-    }
+    stun_message_id (msg, agent->sent_ids[saved_id_idx].id);
+    agent->sent_ids[saved_id_idx].method = stun_message_get_method (msg);
+    agent->sent_ids[saved_id_idx].key = (uint8_t *) key;
+    agent->sent_ids[saved_id_idx].key_len = key_len;
+    memcpy (agent->sent_ids[saved_id_idx].long_term_key, msg->long_term_key,
+        sizeof(msg->long_term_key));
+    agent->sent_ids[saved_id_idx].long_term_valid = msg->long_term_valid;
+    agent->sent_ids[saved_id_idx].valid = TRUE;
   }
 
   msg->key = (uint8_t *) key;
@@ -654,3 +683,8 @@ stun_agent_find_unknowns (StunAgent *agent, const StunMessage * msg,
   stun_debug ("STUN unknown: %u mandatory attribute(s)!\n", count);
   return count;
 }
+
+void stun_agent_set_software (StunAgent *agent, const char *software)
+{
+  agent->software_attribute = software;
+}
