@@ -138,7 +138,10 @@ typedef enum {
  * @STUN_AGENT_USAGE_USE_FINGERPRINT: The agent should add the FINGERPRINT
  * attribute to the STUN messages it creates.
  * @STUN_AGENT_USAGE_ADD_SOFTWARE: The agent should add the SOFTWARE attribute
- * to the STUN messages it creates
+ * to the STUN messages it creates. Calling nice_agent_set_software() will have
+ * the same effect as enabling this Usage. STUN Indications do not have the
+ * SOFTWARE attributes added to them though. The SOFTWARE attribute is only
+ * added for the RFC5389 and WLM2009 compatibility modes.
  * @STUN_AGENT_USAGE_IGNORE_CREDENTIALS: The agent should ignore any credentials
  * in the STUN messages it receives (the MESSAGE-INTEGRITY attribute
  * will never be validated by stun_agent_validate())
@@ -184,6 +187,7 @@ struct stun_agent_t {
   StunAgentSavedIds sent_ids[STUN_AGENT_MAX_SAVED_IDS];
   uint16_t *known_attributes;
   StunAgentUsageFlags usage_flags;
+  const char *software_attribute;
 };
 
 /**
@@ -429,9 +433,70 @@ size_t stun_agent_build_unknown_attributes_error (StunAgent *agent,
  * add the MESSAGE-INTEGRITY and FINGERPRINT attributes if necessary. If the
  * STUN message has a #STUN_REQUEST class, it will save the transaction id of
  * the message in the agent for future matching of the response.
- * Returns: The final size of the message built
+ * <para>See also: stun_agent_forget_transaction()</para>
+ * Returns: The final size of the message built or 0 if an error occured
+ * <note>
+     <para>
+       The return value must always be checked. a value of 0 means the either
+       the buffer's size is too small to contain the finishing attributes
+       (MESSAGE-INTEGRITY, FINGERPRINT), or that there is no more free slots
+       for saving the sent id in the agent's state.
+     </para>
+     <para>
+       Everytime stun_agent_finish_message() is called for a #STUN_REQUEST
+       message, you must make sure to call stun_agent_forget_transaction() in
+       case the response times out and is never received. This is to avoid
+       filling up the #StunAgent's sent ids state preventing any further
+       use of the stun_agent_finish_message()
+     </para>
+   </note>
  */
 size_t stun_agent_finish_message (StunAgent *agent, StunMessage *msg,
    const uint8_t *key, size_t key_len);
 
+/**
+ * stun_agent_forget_transaction:
+ * @agent: The #StunAgent
+ * @id: The #StunTransactionId of the transaction to forget
+ *
+ * This function is used to make the #StunAgent forget about a previously
+ * created transaction.
+ * <para>
+ * This function should be called when a STUN request was previously
+ * created with stun_agent_finish_message() and for which no response was ever
+ * received (timed out). The #StunAgent keeps a list of the sent transactions
+ * in order to validate the responses received. If the response is never received
+ * this will allow the #StunAgent to forget about the timed out transaction and
+ * free its slot for future transactions.
+ * </para>
+ * Returns: %TRUE if the transaction was found, %FALSE otherwise
+ */
+bool stun_agent_forget_transaction (StunAgent *agent, StunTransactionId id);
+
+
+/**
+ * stun_agent_set_software:
+ * @agent: The #StunAgent
+ * @software: The value of the SOFTWARE attribute to add.
+ *
+ * This function will set the value of the SOFTWARE attribute to be added to
+ * STUN requests, responses and error responses.
+ * <para>
+ * Calling this function will automatically enable the addition of the SOFTWARE
+ * attribute for RFC5389 and WLM2009 compatibility modes.
+ * </para>
+ * <note>
+     <para>
+       The @software argument must be in UTF-8 encoding and only the first
+       128 characters will be sent.
+     </para>
+     <para>
+       The value of the @software argument must stay valid throughout the life of
+       the StunAgent's life. Do not free its content.
+     </para>
+   </note>
+ *
+ */
+void stun_agent_set_software (StunAgent *agent, const char *software);
+
 #endif /* _STUN_AGENT_H */
